import requests
from functools import wraps

BASE_URL = 'http://api.are.na/v2'


def paginated(fn):
    @wraps(fn)
    def decorated(*args, page=0, per_page=15, **kwargs):
        params = kwargs.get('params', {})
        params.update({
            'page': page,
            'per': per_page})
        kwargs['params'] = params
        return fn(*args, **kwargs)
    return decorated


class Resource():
    def __init__(self, access_token=None):
        self.access_token = access_token

    def _headers(self, auth):
        if auth:
            if self.access_token is not None:
                return {
                    'Authorization': 'Bearer {}'.format(self.access_token)
                }
            elif self.auth_token is not None:
                return {
                    'X-AUTH-TOKEN': self.auth_token
                }
            raise AttributeError('No access token or auth token is set')
        return {}

    def _get(self, endpoint, params=None, auth=False):
        resp = requests.get(
            ''.join([BASE_URL, self.base_endpoint, endpoint]),
            params=params or {},
            headers=self._headers(auth))
        if resp.status_code != 200:
            resp.raise_for_status()
        return resp.json()

    def _post(self, endpoint, data, params=None):
        resp = requests.post(
            ''.join([BASE_URL, self.base_endpoint, endpoint]),
            params=params or {},
            headers=self._headers(True),
            json=data)
        if resp.status_code != 200:
            resp.raise_for_status()
        return resp.json()

    def _put(self, endpoint, data, params=None):
        resp = requests.put(
            ''.join([BASE_URL, self.base_endpoint, endpoint]),
            params=params or {},
            headers=self._headers(True),
            json=data)
        if resp.status_code != 200:
            resp.raise_for_status()
        return resp.json()

    def _delete(self, endpoint, params=None):
        resp = requests.delete(
            ''.join([BASE_URL, self.base_endpoint, endpoint]),
            params=params or {},
            headers=self._headers(True))
        if resp.status_code != 200:
            resp.raise_for_status()
        return resp.json()



from .resource import Resource, paginated


class Search(Resource):
    base_endpoint = '/search'

    @paginated
    def all(self, query, **kwargs):
        kwargs['params'].update({'q': query})
        return self._get('', params=kwargs['params'])

    @paginated
    def channels(self, query, **kwargs):
        kwargs['params'].update({'q': query})
        return self._get('/channels', params=kwargs['params'])

    @paginated
    def blocks(self, query, **kwargs):
        kwargs['params'].update({'q': query})
        return self._get('/blocks', params=kwargs['params'])

    @paginated
    def users(self, query, **kwargs):
        kwargs['params'].update({'q': query})
        return self._get('/users', params=kwargs['params'])



from .feed import Feed
from .users import Users
from .blocks import Blocks
from .search import Search
from .channels import Channels


class Arena():
    def __init__(self, access_token):
        self.access_token = access_token
        self.users = Users(access_token=access_token)
        self.blocks = Blocks(access_token=access_token)
        self.channels = Channels(access_token=access_token)
        self.feed = Feed(access_token=access_token)
        self.search = Search()



from .search import Search
from .resource import Resource, paginated


class User(Resource):
    base_endpoint = '/users'

    def __init__(self, id, access_token):
        self.id = id
        self.access_token = access_token
        data = self._get('/{}'.format(id))
        for name, val in data.items():
            setattr(self, name, val)

    def channel(self):
        """get the user's channel"""
        return self._get('/{}/channel'.format(self.id), auth=True)

    def channels(self):
        """get the user's channels"""
        return self._get('/{}/channels'.format(self.id), auth=True)

    def following(self):
        """get who/what the user is following"""
        return self._get('/{}/following'.format(self.id), auth=True)

    def followers(self):
        """get the user's followers"""
        return self._get('/{}/followers'.format(self.id), auth=True)


class Users(Resource):
    base_endpoint = '/users'

    def user(self, id):
        """get an existing user"""
        return User(id, access_token=self.access_token)

    @paginated
    def search(self, query, **kwargs):
        return Search().users(query, **kwargs)



from .search import Search
from .resource import Resource, paginated


class Channel(Resource):
    base_endpoint = '/channels'

    def __init__(self, slug, access_token):
        self.slug = slug
        self.access_token = access_token

    @paginated
    def all(self, **kwargs):
        """gets full representation of this channel (paginated)"""
        return self._get('/{}'.format(self.slug), params=kwargs['params'])

    def thumb(self, **kwargs):
        """gets a small representation of this channel"""
        return self._get('/{}/thumb'.format(self.slug))

    @paginated
    def connections(self, **kwargs):
        """get connections for this channel (paginated)"""
        return self._get('/{}/connections'.format(self.slug), params=kwargs['params'])

    @paginated
    def channels(self, **kwargs):
        """get connected channels for this channel (paginated)"""
        return self._get('/{}/channels'.format(self.slug), params=kwargs['params'])

    @paginated
    def contents(self, **kwargs):
        """get only contents for this channel (paginated)"""
        return self._get('/{}/contents'.format(self.slug), params=kwargs['params'])

    def collaborators(self):
        """get only collaborators for this channel"""
        return self._get('/{}/collaborators'.format(self.slug))

    # TODO
    # <http://dev.are.na/documentation/channels#block_45048>
    # def add_collaborator(self):
        # return self._post('{}/collaborators'.format(self.slug)

    def update(self, **kwargs):
        """update a channel's attributes
        can specify:
        - title: str
        - status: one of ['public', 'closed', 'private']
        """
        return self._put('/{}'.format(self.slug), data=kwargs)

    def delete(self):
        """deletes a channel"""
        return self._delete('/{}'.format(self.slug))

    def add_block(self, source=None, content=None):
        """add a new block to the channel.
        one of `source` or `content` must be specified:
        - source: a url
        - content: github-flavored markdown
        """
        if source is not None:
            data = {'source': source}
        elif content is not None:
            data = {'content': content}
        else:
            raise ValueError('One of `source` or `content` must be specified')
        return self._post('/{}/blocks'.format(self.slug), data=data)


class Channels(Resource):
    base_endpoint = '/channels'

    @paginated
    def list(self, **kwargs):
        """list channels (paginated)"""
        return self._get('', params=kwargs['params'])

    def create(self, title, status='public'):
        """creates a new channel"""
        return self._post('', data={
            'title': title,
            'status': status
        })

    def channel(self, slug):
        """get an existing channel"""
        return Channel(slug, access_token=self.access_token)

    @paginated
    def search(self, query, **kwargs):
        return Search().channels(query, **kwargs)



from setuptools import setup, find_packages

setup(
    name='arena',
    version='0.0.2',
    description='python interface to the are.na api',
    url='https://github.com/frnsys/arena',
    author='Francis Tseng (@frnsys)',
    license='MIT',

    packages=find_packages(),
    install_requires=[
        'requests'
    ]
)


from .search import Search
from .resource import Resource, paginated


class Block(Resource):
    base_endpoint = '/blocks'

    def __init__(self, id, access_token):
        self.id = id
        self.access_token = access_token
        data = self._get('/{}'.format(id))
        for name, val in data.items():
            setattr(self, name, val)

    @paginated
    def channels(self, **kwargs):
        """get channels this block is in"""
        return self._get('/{}/channels'.format(self.id), params=kwargs['params'])

    def update(self, **kwargs):
        """update the block's attributes (all are optional):
        - title: str
        - description: markdown
        - content: markdown (for text blocks only)
        """
        return self._put('/{}'.format(self.id), data=kwargs)

    def delete(self):
        """deletes the block"""
        return self._delete('/{}'.format(self.id))


class Blocks(Resource):
    base_endpoint = '/blocks'

    def block(self, id):
        """get an existing block"""
        return Block(id, access_token=self.access_token)

    @paginated
    def search(self, query, **kwargs):
        return Search().blocks(query, **kwargs)
